@@ -29,11 +29,12 @@
 
  */
 
-/* $Id: apc.c 307332 2011-01-10 09:01:23Z pajoye $ */
+/* $Id: apc.c 327370 2012-08-29 21:45:33Z ab $ */
 
 #include "apc.h"
 #include "apc_zend.h"
 #include "apc_cache.h"
+#include "apc_globals.h"
 #include "php.h"
 
 #if HAVE_PCRE || HAVE_BUNDLED_PCRE
@@ -62,12 +63,13 @@ void* apc_emalloc(size_t n TSRMLS_DC)
 
 void* apc_erealloc(void* p, size_t n TSRMLS_DC)
 {
-    p = realloc(p, n);
-    if (p == NULL) {
+    void *new;
+    new = realloc(p, n);
+    if (new == NULL) {
         apc_error("apc_erealloc: realloc failed to allocate %u bytes:" TSRMLS_CC, n);
         return NULL;
     }
-    return p;
+    return new;
 }
 
 void apc_efree(void* p TSRMLS_DC)
@@ -230,7 +232,6 @@ char** apc_tokenize(const char* s, char delim TSRMLS_DC)
 
 /* }}} */
 
-
 /* {{{ apc_win32_restat */
 #ifdef PHP_WIN32
 static int apc_restat(apc_fileinfo_t *fileinfo TSRMLS_DC)
@@ -294,6 +295,20 @@ static int apc_restat(apc_fileinfo_t *fileinfo TSRMLS_DC)
 #endif
 /* }}} */
 
+#define IS_COMMON_WRAPPER(fname, len) \
+     (((len) >= 5 && strncmp((fname), "file:", 5) == 0) || \
+      ((len) >= 5 && strncmp((fname), "http:", 5) == 0) || \
+      ((len) >= 4 && strncmp((fname), "ftp:", 4) == 0)  || \
+      ((len) >= 4 && strncmp((fname), "php:", 4) == 0)  || \
+      ((len) >= 5 && strncmp((fname), "zlib:", 5) == 0) || \
+      ((len) >= 5 && strncmp((fname), "data:", 5) == 0) || \
+      ((len) >= 5 && strncmp((fname), "glob:", 5) == 0) || \
+      ((len) >= 5 && strncmp((fname), "phar:", 5) == 0) || \
+      ((len) >= 5 && strncmp((fname), "ssh2:", 5) == 0) || \
+      ((len) >= 4 && strncmp((fname), "rar:", 4) == 0)  || \
+      ((len) >= 4 && strncmp((fname), "ogg:", 4) == 0)  || \
+      ((len) >= 7 && strncmp((fname), "expect:", 7) == 0))
+
 int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo TSRMLS_DC)
 {
     char** paths = NULL;
@@ -303,18 +318,51 @@ int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fil
     int i;
     php_stream_wrapper *wrapper = NULL;
     char *path_for_open = NULL;
+    int is_user_wrapper = 0;
+    int filename_len = strlen(filename);
 
     assert(filename && fileinfo);
 
+    /* Prevent tries to stat userspace stream wrappers when reading opcodes
+        right after the file was compiled. The core stream wrappers should be
+        already loaded, so looking for them should be ok (not sure about phar) */
+    if (!IS_ABSOLUTE_PATH(filename, filename_len) && 
+        !IS_RELATIVE_PATH(filename, filename_len) &&
+        !IS_COMMON_WRAPPER(filename, filename_len)) {
+        char *p = (char *)filename;
+        int n = 0;
+
+        /* If we're here, it's most likely a userspace stream wrapper. Extract
+            it's protocol and check if it's already been registered. */
+
+        for (; isalnum((int)*p) || *p == '+' || *p == '-' || *p == '.'; p++) {
+            n++;
+        }
+
+        if ((*p == ':') && (n > 1) && (!strncmp("//", p+1, 2) || (n == 4 && !memcmp("data:", path, 5)))) {
+            char *tmp = estrndup(filename, n); 
+
+            if (!zend_hash_exists(php_stream_get_url_stream_wrappers_hash(), tmp, n + 1)) {
+                return -1;
+            }
+            efree(tmp);
+        }
+    }
 
     wrapper = php_stream_locate_url_wrapper(filename, &path_for_open, 0 TSRMLS_CC);
 
-    if(!wrapper || !wrapper->wops || !wrapper->wops->url_stat) {
+    if(!wrapper || !wrapper->wops) {
+        return -1;
+    }
+
+    is_user_wrapper = (NULL != wrapper->wops->label) && (strncmp(wrapper->wops->label, "user-space", 11) == 0);
+
+    if (!is_user_wrapper && !wrapper->wops->url_stat) {
         return -1;
     }
 
     if(wrapper != &php_plain_files_wrapper) {
-        if(APC_URL_STAT(wrapper, path_for_open, &fileinfo->st_buf) == 0) {
+        if(!is_user_wrapper && APC_URL_STAT(wrapper, path_for_open, &fileinfo->st_buf) == 0) {
             fileinfo->fullpath = COPY_IF_CHANGED(path_for_open);
             return apc_restat(fileinfo TSRMLS_CC);
         }
@@ -341,6 +389,14 @@ int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fil
                 break;
             }
         }
+        /* in cli mode PHP explicitly checks the cwd, so we should as well */
+        if(APCG(enable_cli) && !strcmp(sapi_module.name, "cli")) {
+            snprintf(fileinfo->path_buf, sizeof(fileinfo->path_buf), ".%c%s", DEFAULT_SLASH, path_for_open);
+            if (APC_URL_STAT(wrapper, fileinfo->path_buf, &fileinfo->st_buf) == 0) {
+                fileinfo->fullpath = (char*) fileinfo->path_buf;
+                found = 1;
+            }
+        }
     } else {
         /* read cwd and try to fake up fullpath */
         fileinfo->path_buf[0] = '\0';
@@ -464,6 +520,15 @@ void apc_regex_destroy_array(void* p TSRMLS_DC)
 {
     if (p != NULL) {
         apc_regex* regs = (apc_regex*) p;
+/* XXX add more tests and activate when it works properly on all SAPIs*/
+#if 0
+        if (regs->preg) {
+            pefree(regs->preg, 1);
+        }
+        if (regs->nreg) {
+            pefree(regs->nreg, 1);
+        }
+#endif
         apc_efree(regs TSRMLS_CC);
     }
 }
@@ -578,9 +643,9 @@ static unsigned int crc32tab[] = {
     /* 252 */  0xb7bd5c3b, 0x8c3ec470, 0xc0ba6cad, 0xfb39f4e6,
 };
 
-unsigned int apc_crc32(const char* buf, int len)
+unsigned int apc_crc32(const unsigned char* buf, unsigned int len)
 {
-    int i;
+    unsigned int i;
     int k;
     unsigned int crc;
 
@@ -589,6 +654,7 @@ unsigned int apc_crc32(const char* buf, int len)
 
     for (i = 0; i < len; i++) {
         k = (crc ^ buf[i]) & 0x000000FF;
+        assert(k >= 0 && k <= 255);
         crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[k];
     }
 
@@ -618,7 +684,6 @@ static unsigned long crc32gen(int n)
 
 /* }}} */
 
-
 /* {{{ apc_flip_hash() */
 HashTable* apc_flip_hash(HashTable *hash) {
     zval **entry, *data;
@@ -649,7 +714,6 @@ HashTable* apc_flip_hash(HashTable *hash) {
 }
 /* }}} */
 
-
 /*
  * Local variables:
  * tab-width: 4
