@@ -13,11 +13,12 @@
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: Brian Shire <shire@php.net>                                 |
+  |          Xinchen Hui <laruence@php.net>                              |
   +----------------------------------------------------------------------+
 
  */
 
-/* $Id: apc_bin.c 307333 2011-01-10 09:02:13Z pajoye $ */
+/* $Id: apc_bin.c 327151 2012-08-16 15:21:44Z laruence $ */
 
 /* Creates a binary architecture specific output to a string or file containing
  * the current cache contents for both fies and user variables.  This is accomplished
@@ -28,10 +29,10 @@
 #include "apc_globals.h"
 #include "apc_bin.h"
 #include "apc_zend.h"
+#include "apc_php.h"
 #include "apc_sma.h"
 #include "apc_pool.h"
 #include "ext/standard/md5.h"
-#include "ext/standard/crc32.h"
 
 extern apc_cache_t* apc_cache;
 extern apc_cache_t* apc_user_cache;
@@ -40,7 +41,6 @@ extern int _apc_store(char *strkey, int strkey_len, const zval *val, const uint
 
 #define APC_BINDUMP_DEBUG 0
 
-
 #if APC_BINDUMP_DEBUG
 
 #define SWIZZLE(bd, ptr)  \
@@ -82,7 +82,6 @@ extern int _apc_store(char *strkey, int strkey_len, const zval *val, const uint
 
 #endif
 
-
 static void *apc_bd_alloc(size_t size TSRMLS_DC);
 static void apc_bd_free(void *ptr TSRMLS_DC);
 static void *apc_bd_alloc_ex(void *ptr_new, size_t size TSRMLS_DC);
@@ -108,14 +107,12 @@ static void apc_swizzle_arg_info_array(apc_bd_t *bd, zend_llist *ll, const zend_
 static apc_bd_t* apc_swizzle_bd(apc_bd_t* bd, zend_llist *ll TSRMLS_DC);
 static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC);
 
-
 /* {{{ apc_bd_alloc
  *  callback for copy_* functions */
 static void *apc_bd_alloc(size_t size TSRMLS_DC) {
     return apc_bd_alloc_ex(NULL, size TSRMLS_CC);
 } /* }}} */
 
-
 /* {{{ apc_bd_free
  *  callback for copy_* functions */
 static void apc_bd_free(void *ptr TSRMLS_DC) {
@@ -128,7 +125,6 @@ static void apc_bd_free(void *ptr TSRMLS_DC) {
     zend_hash_index_del(&APCG(apc_bd_alloc_list), (ulong)ptr);
 } /* }}} */
 
-
 /* {{{ apc_bd_alloc_ex
  *  set ranges or allocate a block of data from an already (e)malloc'd range.
  *  if ptr_new is not NULL, it will reset the pointer to start at ptr_new,
@@ -157,7 +153,6 @@ static void *apc_bd_alloc_ex(void *ptr_new, size_t size TSRMLS_DC) {
     return rval;
 } /* }}} */
 
-
 /* {{{ _apc_swizzle_ptr */
 static void _apc_swizzle_ptr(apc_bd_t *bd, zend_llist *ll, void **ptr, const char* file, int line TSRMLS_DC) {
     if(*ptr) {
@@ -174,7 +169,6 @@ static void _apc_swizzle_ptr(apc_bd_t *bd, zend_llist *ll, void **ptr, const cha
     }
 } /* }}} */
 
-
 /* {{{ apc_swizzle_op_array */
 static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op_array TSRMLS_DC) {
     uint i;
@@ -191,6 +185,15 @@ static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op
     apc_swizzle_ptr(bd, ll, &op_array->function_name);
     apc_swizzle_ptr(bd, ll, &op_array->filename);
     apc_swizzle_ptr(bd, ll, &op_array->refcount);
+#ifdef ZEND_ENGINE_2_4
+    if (op_array->last_literal) {
+        int i = 0;
+        apc_swizzle_ptr(bd, ll, &(op_array->literals));
+        for (; i<op_array->last_literal; i++) {
+            apc_swizzle_zval(bd, ll, &((op_array->literals[i]).constant) TSRMLS_CC);
+        }
+    }
+#endif
 
     /* swizzle op_array */
     for(i=0; i < op_array->last; i++) {
@@ -204,18 +207,38 @@ static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op
         if(op_array->opcodes[i].op2.op_type == IS_CONST) {
             apc_swizzle_zval(bd, ll, &op_array->opcodes[i].op2.u.constant TSRMLS_CC);
         }
+#else
+        if (op_array->opcodes[i].op1_type == IS_CONST) {
+            apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op1.literal);
+        }
+        if (op_array->opcodes[i].op2_type == IS_CONST) {
+            apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op2.literal);
+        }
+        if (op_array->opcodes[i].result_type == IS_CONST) {
+            apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].result.literal);
+        }
 #endif
         switch (op_array->opcodes[i].opcode) {
+#ifdef ZEND_ENGINE_2_3
+            case ZEND_GOTO:
+#endif
             case ZEND_JMP:
 #ifdef ZEND_ENGINE_2_4
                 apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op1.jmp_addr);
 #else
                 apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op1.u.jmp_addr);
 #endif
+                break;
             case ZEND_JMPZ:
             case ZEND_JMPNZ:
             case ZEND_JMPZ_EX:
             case ZEND_JMPNZ_EX:
+#ifdef ZEND_ENGINE_2_3
+            case ZEND_JMP_SET:
+#endif
+#ifdef ZEND_ENGINE_2_4
+            case ZEND_JMP_SET_VAR:
+#endif
 #ifdef ZEND_ENGINE_2_4
                 apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op2.jmp_addr);
 #else
@@ -262,7 +285,6 @@ static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op
 
 } /* }}} */
 
-
 /* {{{ apc_swizzle_function */
 static void apc_swizzle_function(apc_bd_t *bd, zend_llist *ll, zend_function *func TSRMLS_DC) {
     apc_swizzle_op_array(bd, ll, &func->op_array TSRMLS_CC);
@@ -273,7 +295,6 @@ static void apc_swizzle_function(apc_bd_t *bd, zend_llist *ll, zend_function *fu
 #endif
 } /* }}} */
 
-
 /* {{{ apc_swizzle_class_entry */
 static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_entry *ce TSRMLS_DC) {
 
@@ -294,8 +315,7 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
     apc_swizzle_hashtable(bd, ll, &ce->function_table, (apc_swizzle_cb_t)apc_swizzle_function, 0 TSRMLS_CC);
 #ifdef ZEND_ENGINE_2_4
     if (ce->default_properties_table) {
-        int i;
-
+        apc_swizzle_ptr(bd, ll, &ce->default_properties_table);
         for (i = 0; i < ce->default_properties_count; i++) {
             if (ce->default_properties_table[i]) {
                 apc_swizzle_ptr(bd, ll, &ce->default_properties_table[i]);
@@ -313,8 +333,7 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
 
 #ifdef ZEND_ENGINE_2_4
     if (ce->default_static_members_table) {
-        int i;
-
+        apc_swizzle_ptr(bd, ll, &ce->default_static_members_table);
         for (i = 0; i < ce->default_static_members_count; i++) {
             if (ce->default_static_members_table[i]) {
                 apc_swizzle_ptr(bd, ll, &ce->default_static_members_table[i]);
@@ -361,7 +380,6 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
     }
 } /* }}} */
 
-
 /* {{{ apc_swizzle_property_info */
 static void apc_swizzle_property_info(apc_bd_t *bd, zend_llist *ll, zend_property_info *pi TSRMLS_DC) {
     apc_swizzle_ptr(bd, ll, &pi->name);
@@ -372,7 +390,6 @@ static void apc_swizzle_property_info(apc_bd_t *bd, zend_llist *ll, zend_propert
 #endif
 } /* }}} */
 
-
 /* {{{ apc_swizzle_function_entry */
 static void apc_swizzle_function_entry(apc_bd_t *bd, zend_llist *ll, const zend_function_entry *fe TSRMLS_DC) {
     apc_swizzle_ptr(bd, ll, &fe->fname);
@@ -380,7 +397,6 @@ static void apc_swizzle_function_entry(apc_bd_t *bd, zend_llist *ll, const zend_
     apc_swizzle_ptr(bd, ll, &fe->arg_info);
 } /* }}} */
 
-
 /* {{{ apc_swizzle_arg_info_array */
 static void apc_swizzle_arg_info_array(apc_bd_t *bd, zend_llist *ll, const zend_arg_info* arg_info_array, uint num_args TSRMLS_DC) {
     if(arg_info_array) {
@@ -394,7 +410,6 @@ static void apc_swizzle_arg_info_array(apc_bd_t *bd, zend_llist *ll, const zend_
 
 } /* }}} */
 
-
 /* {{{ apc_swizzle_hashtable */
 static void apc_swizzle_hashtable(apc_bd_t *bd, zend_llist *ll, HashTable *ht, apc_swizzle_cb_t swizzle_cb, int is_ptr TSRMLS_DC) {
     uint i;
@@ -410,6 +425,17 @@ static void apc_swizzle_hashtable(apc_bd_t *bd, zend_llist *ll, HashTable *ht, a
         } else {
             swizzle_cb(bd, ll, (void**)(*bp_prev)->pData TSRMLS_CC);
         }
+#ifdef ZEND_ENGINE_2_4
+        if ((*bp_prev)->nKeyLength) {
+            if (IS_INTERNED((*bp_prev)->arKey)) {
+                /* we should dump that internal string out */
+                char *tmp = apc_bd_alloc((*bp_prev)->nKeyLength TSRMLS_CC);
+                memcpy(tmp, (*bp_prev)->arKey, (*bp_prev)->nKeyLength);
+                (*bp_prev)->arKey = tmp;
+            }
+            apc_swizzle_ptr(bd, ll, &(*bp_prev)->arKey);
+        }
+#endif
         apc_swizzle_ptr(bd, ll, &(*bp_prev)->pData);
         if((*bp_prev)->pDataPtr) {
             apc_swizzle_ptr(bd, ll, &(*bp_prev)->pDataPtr);
@@ -435,7 +461,6 @@ static void apc_swizzle_hashtable(apc_bd_t *bd, zend_llist *ll, HashTable *ht, a
     apc_swizzle_ptr(bd, ll, &ht->arBuckets);
 } /* }}} */
 
-
 /* {{{ apc_swizzle_zval */
 static void apc_swizzle_zval(apc_bd_t *bd, zend_llist *ll, zval *zv TSRMLS_DC) {
 
@@ -446,7 +471,7 @@ static void apc_swizzle_zval(apc_bd_t *bd, zend_llist *ll, zval *zv TSRMLS_DC) {
         zend_hash_index_update(&APCG(copied_zvals), (ulong)zv, (void**)&zv, sizeof(zval*), NULL);
     }
 
-    switch(zv->type & ~IS_CONSTANT_INDEX) {
+    switch(Z_TYPE_P(zv) & IS_CONSTANT_TYPE_MASK) {
         case IS_NULL:
         case IS_LONG:
         case IS_DOUBLE:
@@ -470,15 +495,15 @@ static void apc_swizzle_zval(apc_bd_t *bd, zend_llist *ll, zval *zv TSRMLS_DC) {
     }
 } /* }}} */
 
-
 /* {{{ apc_swizzle_bd */
 static apc_bd_t* apc_swizzle_bd(apc_bd_t* bd, zend_llist *ll TSRMLS_DC) {
-    int count, i;
+    unsigned int i;
+    int count;
     PHP_MD5_CTX context;
     unsigned char digest[16];
     register php_uint32 crc;
     php_uint32 crcinit = 0;
-    char *crc_p;
+    unsigned char *crc_p;
     void ***ptr;
     void ***ptr_list;
 
@@ -512,44 +537,42 @@ static apc_bd_t* apc_swizzle_bd(apc_bd_t* bd, zend_llist *ll TSRMLS_DC) {
         bd->num_swizzled_ptrs = 0;
         bd->swizzled_ptrs = NULL;
     }
-    ((char*)bd)[bd->size-1] = 0;  /* silence null termination for zval strings */
+    ((unsigned char*)bd)[((bd->size >= 1) ? (bd->size-1) : 0)] = 0;  /* silence null termination for zval strings */
     efree(ptr_list);
     bd->swizzled = 1;
 
     /* Generate MD5/CRC32 checksum */
-    for(i=0; i<16; i++) { bd->md5[i] = 0; }
+    memset(bd->md5, 0, 16);
     bd->crc=0;
+
     PHP_MD5Init(&context);
     PHP_MD5Update(&context, (const unsigned char*)bd, bd->size);
     PHP_MD5Final(digest, &context);
-    crc = crcinit^0xFFFFFFFF;
-    crc_p = (char*)bd;
-    for(i=bd->size; i--; ++crc_p) {
-      crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*crc_p)) & 0xFF ];
-    }
-    memcpy(bd->md5, digest, 16);
+    crc = apc_crc32((unsigned char*)bd, bd->size);
+
+    memmove(bd->md5, digest, 16);
     bd->crc = crc;
 
     return bd;
 } /* }}} */
 
-
 /* {{{ apc_unswizzle_bd */
 static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC) {
-    int i;
+    unsigned int i;
     unsigned char md5_orig[16];
     unsigned char digest[16];
     PHP_MD5_CTX context;
     register php_uint32 crc;
     php_uint32 crcinit = 0;
     php_uint32 crc_orig;
-    char *crc_p;
+    unsigned char *crc_p;
 
     /* Verify the md5 or crc32 before we unswizzle */
-    memcpy(md5_orig, bd->md5, 16);
-    for(i=0; i<16; i++) { bd->md5[i] = 0; }
+    memmove(md5_orig, bd->md5, 16);
+    memset(bd->md5, 0, 16);
     crc_orig = bd->crc;
     bd->crc=0;
+
     if(flags & APC_BIN_VERIFY_MD5) {
         PHP_MD5Init(&context);
         PHP_MD5Update(&context, (const unsigned char*)bd, bd->size);
@@ -560,11 +583,7 @@ static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC) {
         }
     }
     if(flags & APC_BIN_VERIFY_CRC32) {
-        crc = crcinit^0xFFFFFFFF;
-        crc_p = (char*)bd;
-        for(i=bd->size; i--; ++crc_p) {
-          crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*crc_p)) & 0xFF ];
-        }
+        crc = apc_crc32((unsigned char*)bd, bd->size);
         if(crc_orig != crc) {
             apc_error("CRC32 checksum of binary dump failed." TSRMLS_CC);
             return -1;
@@ -589,7 +608,6 @@ static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC) {
     return 0;
 } /* }}} */
 
-
 /* {{{ apc_bin_checkfilter */
 static int apc_bin_checkfilter(HashTable *filter, const char *key, uint key_len) {
     zval **zptr;
@@ -649,16 +667,14 @@ static inline void apc_bin_fixup_class_entry(zend_class_entry *ce) {
 
 /* {{{ apc_bin_dump */
 apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
-
-    int i;
     uint fcount;
     slot_t *sp;
     apc_bd_entry_t *ep;
-    int count=0;
+    int i, count=0;
     apc_bd_t *bd;
     zend_llist ll;
     zend_function *efp, *sfp;
-    long size=0;
+    size_t size=0;
     apc_context_t ctxt;
     void *pool_ptr;
 
@@ -698,7 +714,7 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
 
     size += sizeof(apc_bd_t) +1;  /* +1 for null termination */
     bd = emalloc(size);
-    bd->size = size;
+    bd->size = (unsigned int)size;
     pool_ptr = emalloc(sizeof(apc_pool));
     apc_bd_alloc_ex(pool_ptr, sizeof(apc_pool) TSRMLS_CC);
     ctxt.pool = apc_pool_create(APC_UNPOOL, apc_bd_alloc, apc_bd_free, NULL, NULL TSRMLS_CC);  /* ideally the pool wouldn't be alloc'd as part of this */
@@ -706,7 +722,7 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
         apc_warning("Unable to allocate memory for pool." TSRMLS_CC);
         return NULL;
     }
-    ctxt.copy = APC_COPY_IN_USER;  /* avoid stupid ALLOC_ZVAL calls here, hack */
+    ctxt.copy = APC_COPY_IN_OPCODE; /* avoid stupid ALLOC_ZVAL calls here, hack */
     apc_bd_alloc_ex((void*)((long)bd + sizeof(apc_bd_t)), bd->size - sizeof(apc_bd_t) -1 TSRMLS_CC);
     bd->num_entries = count;
     bd->entries = apc_bd_alloc_ex(NULL, sizeof(apc_bd_entry_t) * count TSRMLS_CC);
@@ -723,13 +739,39 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
                 ep->val.user.info = apc_bd_alloc(sp->value->data.user.info_len TSRMLS_CC);
                 memcpy(ep->val.user.info, sp->value->data.user.info, sp->value->data.user.info_len);
                 ep->val.user.info_len = sp->value->data.user.info_len;
-                ep->val.user.val = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt TSRMLS_CC);
+                if ((Z_TYPE_P(sp->value->data.user.val) == IS_ARRAY && APCG(serializer))
+                        || Z_TYPE_P(sp->value->data.user.val) == IS_OBJECT) {
+                    /* avoiding hash copy, hack */
+                    uint type = Z_TYPE_P(sp->value->data.user.val);
+                    Z_TYPE_P(sp->value->data.user.val) = IS_STRING;
+                    ep->val.user.val = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt TSRMLS_CC);
+                    Z_TYPE_P(ep->val.user.val) = IS_OBJECT;
+                    sp->value->data.user.val->type = type;
+                } else if (Z_TYPE_P(sp->value->data.user.val) == IS_ARRAY && !APCG(serializer)) {
+                    /* this is a little complicated, we have to unserialize it first, then serialize it again */
+                    zval *garbage;
+                    ctxt.copy = APC_COPY_OUT_USER;
+                    garbage = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt TSRMLS_CC);
+                    APCG(serializer) = apc_find_serializer("php" TSRMLS_CC);
+                    ctxt.copy = APC_COPY_IN_USER;
+                    ep->val.user.val = apc_copy_zval(NULL, garbage, &ctxt TSRMLS_CC);
+                    ep->val.user.val->type = IS_OBJECT;
+                    /* a memleak can not be avoided: zval_ptr_dtor(&garbage); */
+                    APCG(serializer) = NULL;
+                    ctxt.copy = APC_COPY_IN_OPCODE;
+                } else {
+                    ep->val.user.val = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt TSRMLS_CC);
+                }
                 ep->val.user.ttl = sp->value->data.user.ttl;
 
                 /* swizzle pointers */
                 apc_swizzle_ptr(bd, &ll, &bd->entries[count].val.user.info);
                 zend_hash_clean(&APCG(copied_zvals));
-                apc_swizzle_zval(bd, &ll, bd->entries[count].val.user.val TSRMLS_CC);
+                if (ep->val.user.val->type == IS_OBJECT) {
+                    apc_swizzle_ptr(bd, &ll, &bd->entries[count].val.user.val->value.str.val);
+                } else {
+                    apc_swizzle_zval(bd, &ll, bd->entries[count].val.user.val TSRMLS_CC);
+                }
                 apc_swizzle_ptr(bd, &ll, &bd->entries[count].val.user.val);
 
                 count++;
@@ -828,10 +870,8 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
     return bd;
 } /* }}} */
 
-
 /* {{{ apc_bin_load */
 int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
-
     apc_bd_entry_t *ep;
     uint i, i2;
     int ret;
@@ -957,8 +997,25 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
 
                 break;
             case APC_CACHE_KEY_USER:
-                ctxt.copy = APC_COPY_IN_USER;
-                _apc_store(ep->val.user.info, ep->val.user.info_len, ep->val.user.val, ep->val.user.ttl, 0 TSRMLS_CC);
+                {
+                    zval *data;
+                    uint use_copy = 0;
+                    switch (Z_TYPE_P(ep->val.user.val)) {
+                        case IS_OBJECT:
+                            ctxt.copy = APC_COPY_OUT_USER;
+                            data = apc_copy_zval(NULL, ep->val.user.val, &ctxt TSRMLS_CC);
+                            use_copy = 1;
+                        break;
+                        default:
+                            data = ep->val.user.val;
+                        break;
+                    }
+                    ctxt.copy = APC_COPY_IN_USER;
+                    _apc_store(ep->val.user.info, ep->val.user.info_len, data, ep->val.user.ttl, 0 TSRMLS_CC);
+                    if (use_copy) {
+                        zval_ptr_dtor(&data);
+                    }
+                }
                 break;
             default:
                 break;
@@ -979,7 +1036,6 @@ failure:
     return -1;
 } /* }}} */
 
-
 /*
  * Local variables:
  * tab-width: 4
