@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_cache.c 309488 2011-03-20 23:59:42Z pajoye $ */
+/* $Id: apc_cache.c 327066 2012-08-12 07:48:48Z laruence $ */
 
 #include "apc_cache.h"
 #include "apc_zend.h"
@@ -99,7 +99,6 @@ static inline unsigned long murmurhash(const char *skey, size_t keylen)
 #endif
 /* }}} */
 
-
 /* {{{ make_prime */
 static int const primes[] = {
   257, /*   256 */
@@ -314,9 +313,14 @@ apc_cache_t* apc_cache_create(int size_hint, int gc_ttl, int ttl TSRMLS_DC)
 void apc_cache_destroy(apc_cache_t* cache TSRMLS_DC)
 {
     DESTROY_LOCK(cache->header->lock);
-#ifdef NONBLOCKING_LOCK_AVAILABLE
+#if NONBLOCKING_LOCK_AVAILABLE
     DESTROY_LOCK(cache->header->wrlock);
 #endif
+
+	/* XXX this is definitely a leak, but freeing this causes all the apache
+		children to freeze. It might be because the segment is shared between
+		several processes. To figure out is how to free this safely. */
+    /*apc_sma_free(cache->shmaddr TSRMLS_CC);*/
     apc_efree(cache TSRMLS_CC);
 }
 /* }}} */
@@ -535,7 +539,6 @@ int *apc_cache_insert_mult(apc_cache_t* cache, apc_cache_key_t* keys, apc_cache_
 }
 /* }}} */
 
-
 /* {{{ apc_cache_user_insert */
 int apc_cache_user_insert(apc_cache_t* cache, apc_cache_key_t key, apc_cache_entry_t* value, apc_context_t* ctxt, time_t t, int exclusive TSRMLS_DC)
 {
@@ -944,7 +947,7 @@ int apc_cache_make_file_key(apc_cache_key_t* key,
 
     len = strlen(filename);
     if(APCG(fpstat)==0) {
-        if(IS_ABSOLUTE_PATH(filename,len)) {
+        if(IS_ABSOLUTE_PATH(filename,len) || strstr(filename, "://")) {
             key->data.fpfile.fullpath = filename;
             key->data.fpfile.fullpath_len = len;
             key->h = string_nhash_8(key->data.fpfile.fullpath, key->data.fpfile.fullpath_len);
@@ -1169,7 +1172,7 @@ apc_cache_entry_t* apc_cache_make_user_entry(const char* info, int info_len, con
 }
 /* }}} */
 
-/* {{{ */
+/* {{{ apc_cache_link_info */
 static zval* apc_cache_link_info(apc_cache_t *cache, slot_t* p TSRMLS_DC)
 {
     zval *link = NULL;
@@ -1290,7 +1293,9 @@ zval* apc_cache_info(apc_cache_t* cache, zend_bool limited TSRMLS_DC)
                 add_next_index_zval(list, link);
                 j++;
             }
-            add_next_index_long(slots, j);
+            if(j != 0) {
+                add_index_long(slots, (ulong)i, j);
+            }
         }
 
         /* For each slot pending deletion */
