@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_cache.h 309327 2011-03-17 07:33:32Z gopalv $ */
+/* $Id: apc_cache.h 326865 2012-07-29 14:19:27Z ab $ */
 
 #ifndef APC_CACHE_H
 #define APC_CACHE_H
@@ -133,6 +133,7 @@ typedef union _apc_cache_entry_value_t {
         apc_function_t* functions;  /* array of apc_function_t's */
         apc_class_t* classes;       /* array of apc_class_t's */
         long halt_offset;           /* value of __COMPILER_HALT_OFFSET__ for the file */
+        zend_uint *exec_refcount;   /* refcount member of zend_op_array refreshed before execution */
     } file;
     struct {
         char *info;
@@ -287,7 +288,7 @@ extern apc_cache_entry_t* apc_cache_make_file_entry(const char* filename,
                                                     TSRMLS_DC);
 
 
-zend_bool apc_compile_cache_entry(apc_cache_key_t key, zend_file_handle* h, int type, time_t t, zend_op_array** op_array_pp, apc_cache_entry_t** cache_entry_pp TSRMLS_DC);
+zend_bool apc_compile_cache_entry(apc_cache_key_t *key, zend_file_handle* h, int type, time_t t, zend_op_array** op_array_pp, apc_cache_entry_t** cache_entry_pp TSRMLS_DC);
 
 /*
  * apc_cache_make_user_entry creates an apc_cache_entry_t object given an info string
