@@ -28,19 +28,16 @@
 
  */
 
-/* $Id: apc_compile.c 307314 2011-01-10 00:06:29Z pajoye $ */
+/* $Id: apc_compile.c 327148 2012-08-16 14:11:18Z laruence $ */
 
 #include "apc_compile.h"
 #include "apc_globals.h"
 #include "apc_zend.h"
+#include "apc_php.h"
 #include "apc_string.h"
 #include "ext/standard/php_var.h"
 #include "ext/standard/php_smart_str.h"
 
-#ifndef IS_CONSTANT_TYPE_MASK
-#define IS_CONSTANT_TYPE_MASK (~IS_CONSTANT_INDEX)
-#endif
-
 typedef void* (*ht_copy_fun_t)(void*, void*, apc_context_t* TSRMLS_DC);
 //typedef void  (*ht_free_fun_t)(void*, apc_context_t*);
 typedef int (*ht_check_copy_fun_t)(Bucket*, va_list);
@@ -49,6 +46,12 @@ typedef void (*ht_fixup_fun_t)(Bucket*, zend_class_entry*, zend_class_entry*);
 
 #define CHECK(p) { if ((p) == NULL) return NULL; }
 
+enum {
+    APC_FREE_HASHTABLE_FUNCS,
+    APC_FREE_HASHTABLE_PROPS,
+    APC_FREE_HASHTABLE_STATIC_PROPS,
+};
+
 /* {{{ internal function declarations */
 
 static zend_function* my_bitwise_copy_function(zend_function*, zend_function*, apc_context_t* TSRMLS_DC);
@@ -60,7 +63,9 @@ static zend_function* my_bitwise_copy_function(zend_function*, zend_function*, a
  */
 static zval** my_copy_zval_ptr(zval**, const zval**, apc_context_t* TSRMLS_DC);
 static zval* my_copy_zval(zval*, const zval*, apc_context_t* TSRMLS_DC);
+#ifndef ZEND_ENGINE_2_4
 static znode* my_copy_znode(znode*, znode*, apc_context_t* TSRMLS_DC);
+#endif
 static zend_op* my_copy_zend_op(zend_op*, zend_op*, apc_context_t* TSRMLS_DC);
 static zend_function* my_copy_function(zend_function*, zend_function*, apc_context_t* TSRMLS_DC);
 static zend_function_entry* my_copy_function_entry(zend_function_entry*, const zend_function_entry*, apc_context_t* TSRMLS_DC);
@@ -93,9 +98,11 @@ static void my_fixup_property_info( Bucket *p, zend_class_entry *src, zend_class
  * defined/overridden in the 'current' class and not inherited.
  */
 static int my_check_copy_function(Bucket* src, va_list args);
-static int my_check_copy_default_property(Bucket* p, va_list args);
 static int my_check_copy_property_info(Bucket* src, va_list args);
+#ifndef ZEND_ENGINE_2_4
+static int my_check_copy_default_property(Bucket* p, va_list args);
 static int my_check_copy_static_member(Bucket* src, va_list args);
+#endif
 static int my_check_copy_constant(Bucket* src, va_list args);
 
 /* }}} */
@@ -242,7 +249,7 @@ static zval* my_serialize_object(zval* dst, const zval* src, apc_context_t* ctxt
     }
 
     if(serialize((unsigned char**)&buf.c, &buf.len, src, config TSRMLS_CC)) {
-        dst->type = src->type & ~IS_CONSTANT_INDEX;
+        dst->type = src->type & ~IS_CONSTANT_INDEX; 
         dst->value.str.len = buf.len;
         CHECK(dst->value.str.val = apc_pmemcpy(buf.c, (buf.len + 1), pool TSRMLS_CC));
     }
@@ -275,18 +282,22 @@ static zval* my_unserialize_object(zval* dst, const zval* src, apc_context_t* ct
 }
 /* }}} */
 
+/* {{{ apc_string_pmemcpy */
 static char *apc_string_pmemcpy(char *str, size_t len, apc_pool* pool TSRMLS_DC)
 {	
 #ifdef ZEND_ENGINE_2_4
+#ifndef ZTS
     if (pool->type != APC_UNPOOL) {
-        char * ret = apc_new_interned_string(str, len TSRMLS_CC);
+        char * ret = (char*)apc_new_interned_string((const char*)str, len TSRMLS_CC);
         if (ret) {
             return ret;
         }
     }
+#endif
 #endif
     return apc_pmemcpy(str, len, pool TSRMLS_CC);
 }
+/* }}} */
 
 /* {{{ my_copy_zval */
 static APC_HOTSPOT zval* my_copy_zval(zval* dst, const zval* src, apc_context_t* ctxt TSRMLS_DC)
@@ -365,6 +376,12 @@ static APC_HOTSPOT zval* my_copy_zval(zval* dst, const zval* src, apc_context_t*
             dst = my_unserialize_object(dst, src, ctxt TSRMLS_CC);
         }
         break;
+#ifdef ZEND_ENGINE_2_4
+    case IS_CALLABLE:
+        /* XXX implement this */
+        assert(0);
+        break;
+#endif
 
     default:
         assert(0);
@@ -396,7 +413,11 @@ static zend_op* my_copy_zend_op(zend_op* dst, zend_op* src, apc_context_t* ctxt
 
     my_check_znode(dst->result_type & ~EXT_TYPE_UNUSED, ctxt TSRMLS_CC);
     my_check_znode(dst->op1_type, ctxt TSRMLS_CC);
-    my_check_znode(dst->op2_type, ctxt TSRMLS_CC);
+
+    /* ZEND_BIND_TRAITS opcode don't set the op2 at all, so ommit its check */
+    if (dst->opcode != ZEND_BIND_TRAITS) {
+        my_check_znode(dst->op2_type, ctxt TSRMLS_CC);
+    }
 
     return dst;
 }
@@ -551,7 +572,7 @@ static zend_property_info* my_copy_property_info(zend_property_info* dst, zend_p
          * string of the form:
          *      \0<classname>\0<membername>\0
          */
-        CHECK((dst->name = apc_string_pmemcpy(src->name, src->name_length+1, pool TSRMLS_CC)));
+        CHECK((dst->name = apc_string_pmemcpy((char *)src->name, src->name_length+1, pool TSRMLS_CC)));
     }
 
 #if defined(ZEND_ENGINE_2) && PHP_MINOR_VERSION > 0
@@ -635,6 +656,7 @@ zend_class_entry* apc_copy_class_entry(zend_class_entry* dst, zend_class_entry*
 {
     return my_copy_class_entry(dst, src, ctxt TSRMLS_CC);
 }
+/* }}} */
 
 /* {{{ my_copy_class_entry */
 static zend_class_entry* my_copy_class_entry(zend_class_entry* dst, zend_class_entry* src, apc_context_t* ctxt TSRMLS_DC)
@@ -705,6 +727,43 @@ static zend_class_entry* my_copy_class_entry(zend_class_entry* dst, zend_class_e
 	dst->__callstatic = NULL;
 #endif
 
+#ifdef ZEND_ENGINE_2_4
+    if (src->trait_aliases) {
+        int num_trait_aliases = 0;
+
+        /* get how much trait aliases we've got */
+        while (src->trait_aliases[num_trait_aliases]) {num_trait_aliases++;}
+
+        /* copy all the trait aliases */
+        CHECK(dst->trait_aliases =
+                (zend_trait_alias **) apc_pool_alloc(pool, sizeof(zend_trait_alias *)*(num_trait_aliases+1)));
+
+        i = 0;
+        while (src->trait_aliases[i] && i < num_trait_aliases) {
+            dst->trait_aliases[i] = apc_copy_trait_alias(NULL, src->trait_aliases[i], ctxt TSRMLS_CC);
+            i++;
+        }
+        dst->trait_aliases[i] = NULL;
+    }
+    if (src->trait_precedences) {
+        int num_trait_precedences = 0;
+
+        /* get how much trait precedences we've got */
+        while (src->trait_precedences[num_trait_precedences]) {num_trait_precedences++;}
+
+        /* copy all the trait precedences */
+        CHECK(dst->trait_precedences =
+                (zend_trait_alias **) apc_pool_alloc(pool, sizeof(zend_trait_alias *)*(num_trait_precedences+1)));
+
+        i = 0;
+        while (src->trait_precedences[i] && i < num_trait_precedences) {
+            dst->trait_precedences[i] = apc_copy_trait_precedence(NULL, src->trait_precedences[i], ctxt TSRMLS_CC);
+            i++;
+        }
+        dst->trait_precedences[i] = NULL;
+    }
+#endif
+
     /* unset function proxies */
     dst->serialize_func = NULL;
     dst->unserialize_func = NULL;
@@ -831,6 +890,57 @@ static zend_class_entry* my_copy_class_entry(zend_class_entry* dst, zend_class_e
 }
 /* }}} */
 
+/* {{{ my_free_hashtable */
+static void my_free_hashtable(HashTable* ht, int type TSRMLS_DC)
+{
+    Bucket *curr = NULL;
+    Bucket *prev = NULL;
+
+    assert(ht != NULL);
+
+    prev = ht->pListHead;
+    while (prev != NULL) {
+        curr = prev;
+        prev = prev->pListNext;
+
+#ifdef ZEND_ENGINE_2_4
+        if (!IS_INTERNED(curr->arKey)) {
+            apc_php_free((char*)curr->arKey TSRMLS_CC);
+        } 
+#else
+        apc_php_free(curr->arKey TSRMLS_CC);
+#endif
+        switch (type) {
+            case APC_FREE_HASHTABLE_FUNCS:
+                do {
+#if 0
+                    /* XXX need to check where else the method is referenced,
+                        the refcount leak is still there */
+                    zend_function *fn;
+                    fn = (zend_function *)curr->pData;
+
+                    apc_free_op_array_after_execution(&fn->op_array TSRMLS_CC);
+#endif
+                    efree(curr->pData);
+                } while (0);
+                break;
+
+            case APC_FREE_HASHTABLE_PROPS:
+            case APC_FREE_HASHTABLE_STATIC_PROPS:
+                efree(curr->pData);
+                break;
+        }
+
+        /* XXX check if this must be ignored for any class method */
+        if (type != APC_FREE_HASHTABLE_FUNCS) {
+            apc_php_free(curr TSRMLS_CC);
+        }
+    }
+
+    apc_php_free(ht->arBuckets TSRMLS_CC);
+}
+/* }}} */
+
 /* {{{ my_copy_hashtable_ex */
 static APC_HOTSPOT HashTable* my_copy_hashtable_ex(HashTable* dst,
                                     HashTable* src TSRMLS_DC,
@@ -886,21 +996,22 @@ static APC_HOTSPOT HashTable* my_copy_hashtable_ex(HashTable* dst,
             CHECK((newp = (Bucket*) apc_pmemcpy(curr, sizeof(Bucket), pool TSRMLS_CC)));
         } else if (IS_INTERNED(curr->arKey)) {
             CHECK((newp = (Bucket*) apc_pmemcpy(curr, sizeof(Bucket), pool TSRMLS_CC)));
+#ifndef ZTS
         } else if (pool->type != APC_UNPOOL) {
             char *arKey;
 
-            arKey = apc_new_interned_string(curr->arKey, curr->nKeyLength TSRMLS_CC);
+            CHECK((newp = (Bucket*) apc_pmemcpy(curr, sizeof(Bucket), pool TSRMLS_CC)));
+            arKey = (char *)apc_new_interned_string(curr->arKey, curr->nKeyLength TSRMLS_CC);
             if (!arKey) {
-                CHECK((newp = (Bucket*) apc_pmemcpy(curr, (sizeof(Bucket) + curr->nKeyLength), pool TSRMLS_CC)));
-                newp->arKey = ((char*)newp) + sizeof(Bucket);
+                CHECK((newp->arKey = (char*) apc_pmemcpy(curr->arKey, curr->nKeyLength, pool TSRMLS_CC)));
             } else {
-                CHECK((newp = (Bucket*) apc_pmemcpy(curr, sizeof(Bucket), pool TSRMLS_CC)));
                 newp->arKey = arKey;
             }
+#endif
         } else {
-            CHECK((newp = (Bucket*) apc_pmemcpy(curr, (sizeof(Bucket) + curr->nKeyLength), pool TSRMLS_CC)));
-            newp->arKey = ((char*)newp) + sizeof(Bucket);
-        }        
+            CHECK((newp = (Bucket*) apc_pmemcpy(curr, sizeof(Bucket), pool TSRMLS_CC)));
+            CHECK((newp->arKey = (char*) apc_pmemcpy(curr->arKey, curr->nKeyLength, pool TSRMLS_CC)));
+        }
 #else
         CHECK((newp = (Bucket*) apc_pmemcpy(curr,
                                   (sizeof(Bucket) + curr->nKeyLength - 1),
@@ -947,6 +1058,8 @@ static APC_HOTSPOT HashTable* my_copy_hashtable_ex(HashTable* dst,
 
     dst->pListTail = newp;
 
+    zend_hash_internal_pointer_reset(dst);
+
     return dst;
 }
 /* }}} */
@@ -1015,6 +1128,9 @@ static void apc_fixup_op_array_jumps(zend_op_array *dst, zend_op_array *src )
 #ifdef ZEND_ENGINE_2_3
             case ZEND_JMP_SET:
 #endif
+#ifdef ZEND_ENGINE_2_4
+            case ZEND_JMP_SET_VAR:
+#endif
 #ifdef ZEND_ENGINE_2_4
                 zo->op2.jmp_addr = dst->opcodes + (zo->op2.jmp_addr - src->opcodes);
 #else
@@ -1124,6 +1240,9 @@ zend_op_array* apc_copy_op_array(zend_op_array* dst, zend_op_array* src, apc_con
             case ZEND_JMPNZ_EX:
 #ifdef ZEND_ENGINE_2_3
             case ZEND_JMP_SET:
+#endif
+#ifdef ZEND_ENGINE_2_4
+            case ZEND_JMP_SET_VAR:
 #endif
                 if(flags != NULL) {
                     flags->has_jumps = 1;
@@ -1259,6 +1378,7 @@ zend_op_array* apc_copy_op_array(zend_op_array* dst, zend_op_array* src, apc_con
                             dzo->op1.literal = (zend_literal*) apc_pool_alloc(pool, sizeof(zend_literal));
                             Z_STRLEN_P(dzo->op1.zv) = strlen(fullpath);
                             Z_STRVAL_P(dzo->op1.zv) = apc_pstrdup(fullpath, pool TSRMLS_CC);
+                            Z_TYPE_P(dzo->op1.zv) = IS_STRING;
                             Z_SET_REFCOUNT_P(dzo->op1.zv, 2);
                             Z_SET_ISREF_P(dzo->op1.zv);
                             dzo->op1.literal->hash_value = zend_hash_func(Z_STRVAL_P(dzo->op1.zv), Z_STRLEN_P(dzo->op1.zv)+1);
@@ -1305,7 +1425,7 @@ zend_op_array* apc_copy_op_array(zend_op_array* dst, zend_op_array* src, apc_con
         for(i = 0; i <  src->last_var; i++) dst->vars[i].name = NULL;
 
         for(i = 0; i <  src->last_var; i++) {
-            CHECK(dst->vars[i].name = apc_string_pmemcpy(src->vars[i].name,
+            CHECK(dst->vars[i].name = apc_string_pmemcpy((char *)src->vars[i].name,
                                 src->vars[i].name_len + 1,
                                 pool TSRMLS_CC));
         }
@@ -1321,7 +1441,6 @@ zend_op_array* apc_copy_op_array(zend_op_array* dst, zend_op_array* src, apc_con
 }
 /* }}} */
 
-
 /* {{{ apc_copy_new_functions */
 apc_function_t* apc_copy_new_functions(int old_count, apc_context_t* ctxt TSRMLS_DC)
 {
@@ -1581,6 +1700,9 @@ static int my_prepare_op_array_for_execution(zend_op_array* dst, zend_op_array*
 #ifdef ZEND_ENGINE_2_3
                 case ZEND_JMP_SET:
 #endif
+#ifdef ZEND_ENGINE_2_4
+                case ZEND_JMP_SET_VAR:
+#endif
 #ifdef ZEND_ENGINE_2_4
                     dzo->op2.jmp_addr = dst->opcodes +
                                             (zo->op2.jmp_addr - src->opcodes);
@@ -1638,7 +1760,6 @@ zend_op_array* apc_copy_op_array_for_execution(zend_op_array* dst, zend_op_array
     memcpy(dst, src, sizeof(src[0]));
     dst->static_variables = my_copy_static_variables(src, ctxt TSRMLS_CC);
 
-    /* memory leak */
     dst->refcount = apc_pmemcpy(src->refcount,
                                       sizeof(src->refcount[0]),
                                       ctxt->pool TSRMLS_CC);
@@ -1649,6 +1770,15 @@ zend_op_array* apc_copy_op_array_for_execution(zend_op_array* dst, zend_op_array
 }
 /* }}} */
 
+/* {{{ apc_free_op_array_after_execution */
+void apc_free_op_array_after_execution(zend_op_array *src TSRMLS_DC)
+{
+    if (NULL != src->refcount) {
+        apc_php_free(src->refcount TSRMLS_CC);
+    }
+}
+/* }}} */
+
 /* {{{ apc_copy_function_for_execution */
 zend_function* apc_copy_function_for_execution(zend_function* src, apc_context_t* ctxt TSRMLS_DC)
 {
@@ -1664,11 +1794,25 @@ zend_function* apc_copy_function_for_execution(zend_function* src, apc_context_t
 /* {{{ apc_copy_function_for_execution_ex */
 zend_function* apc_copy_function_for_execution_ex(void *dummy, zend_function* src, apc_context_t* ctxt TSRMLS_DC)
 {
-    if(src->type==ZEND_INTERNAL_FUNCTION || src->type==ZEND_OVERLOADED_FUNCTION) return src;
+    if(src->type==ZEND_INTERNAL_FUNCTION || src->type==ZEND_OVERLOADED_FUNCTION) {
+        return src;
+    }
+
     return apc_copy_function_for_execution(src, ctxt TSRMLS_CC);
 }
 /* }}} */
 
+/* {{{ apc_free_function_after_execution */
+void apc_free_function_after_execution(zend_function* src TSRMLS_DC)
+{
+    if(src->type==ZEND_INTERNAL_FUNCTION || src->type==ZEND_OVERLOADED_FUNCTION) {
+        return;
+    }
+
+    apc_free_op_array_after_execution(&src->op_array TSRMLS_CC);
+}
+/* }}} */
+
 /* {{{ apc_copy_class_entry_for_execution */
 zend_class_entry* apc_copy_class_entry_for_execution(zend_class_entry* src, apc_context_t* ctxt TSRMLS_DC)
 {
@@ -1694,6 +1838,7 @@ zend_class_entry* apc_copy_class_entry_for_execution(zend_class_entry* src, apc_
     /* Deep-copy the class properties, because they will be modified */
 
 #ifdef ZEND_ENGINE_2_4
+    dst->name = apc_string_pmemcpy((char*)src->name, src->name_length+1, ctxt->pool TSRMLS_CC); 
 	dst->default_properties_count = src->default_properties_count;
     if (src->default_properties_count) {
         dst->default_properties_table = (zval**) apc_php_malloc((sizeof(zval*) * src->default_properties_count) TSRMLS_CC);
@@ -1717,7 +1862,6 @@ zend_class_entry* apc_copy_class_entry_for_execution(zend_class_entry* src, apc_
 
     /* For derived classes, we must also copy the function hashtable (although
      * we can merely bitwise copy the functions it contains) */
-
     my_copy_hashtable(&dst->function_table,
                       &src->function_table,
                       (ht_copy_fun_t) apc_copy_function_for_execution_ex,
@@ -1786,6 +1930,41 @@ zend_class_entry* apc_copy_class_entry_for_execution(zend_class_entry* src, apc_
     }
 #endif
 
+#ifdef ZEND_ENGINE_2_4
+    if (src->trait_aliases) {
+        int num_trait_aliases = 0;
+
+        /* get how much trait aliases we've got */
+        while (src->trait_aliases[num_trait_aliases]) {num_trait_aliases++;}
+
+        /* copy all the trait aliases */
+        CHECK(dst->trait_aliases = (zend_trait_alias **) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_alias *)*(num_trait_aliases+1)));
+        i = 0;
+        while (src->trait_aliases[i]) {
+            dst->trait_aliases[i] = apc_copy_trait_alias_for_execution(src->trait_aliases[i], ctxt TSRMLS_CC);
+            i++;
+        }
+        dst->trait_aliases[i] = NULL;
+    } 
+    if (src->trait_precedences) {
+        int num_trait_precedences = 0;
+
+        /* get how much trait precedences we've got */
+        while (src->trait_precedences[num_trait_precedences]) {num_trait_precedences++;}
+
+        /* copy all the trait precedences */
+        CHECK(dst->trait_precedences = 
+                (zend_trait_alias **) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_alias *)*(num_trait_precedences+1)));
+        i = 0;
+        while (src->trait_precedences[i]) {
+            dst->trait_precedences[i] =
+                apc_copy_trait_precedence_for_execution(src->trait_precedences[i], ctxt TSRMLS_CC);
+            i++;
+        }
+        dst->trait_precedences[i] = NULL;
+    } 
+#endif
+
     return dst;
 }
 /* }}} */
@@ -1801,6 +1980,11 @@ void apc_free_class_entry_after_execution(zend_class_entry* src TSRMLS_DC)
     /* my_destroy_hashtable() does not play nice with refcounts */
 
 #ifdef ZEND_ENGINE_2_4
+#if 0
+    if (!IS_INTERNED(src->name)) {
+        apc_php_free(src->name TSRMLS_CC);
+    }
+#endif
     if (src->default_static_members_table) {
        int i;
 
@@ -1839,6 +2023,17 @@ void apc_free_class_entry_after_execution(zend_class_entry* src TSRMLS_DC)
 #endif
     zend_hash_clean(&src->constants_table);
 
+#ifdef ZEND_ENGINE_2_4
+#if 0
+    /* XXX function freeing seems to be still a fail even in 5.4 as well, so let it down for now */
+    my_free_hashtable(&src->function_table, APC_FREE_HASHTABLE_FUNCS TSRMLS_CC);
+
+    my_free_hashtable(&src->properties_info, APC_FREE_HASHTABLE_PROPS TSRMLS_CC);
+    my_free_hashtable(&src->constants_table, APC_FREE_HASHTABLE_STATIC_PROPS TSRMLS_CC);
+    src->trait_aliases = NULL; /* XXX free all the traits stuff */
+#endif
+#endif
+
     /* TODO: more cleanup */
 }
 /* }}} */
@@ -1883,8 +2078,6 @@ void apc_do_halt_compiler_register(const char *filename, long halt_offset TSRMLS
 }
 /* }}} */
 
-
-
 /* {{{ my_fixup_function */
 static void my_fixup_function(Bucket *p, zend_class_entry *src, zend_class_entry *dst)
 {
@@ -1972,7 +2165,6 @@ static void my_fixup_hashtable(HashTable *ht, ht_fixup_fun_t fixup, zend_class_e
 }
 /* }}} */
 
-
 /* {{{ my_check_copy_function */
 static int my_check_copy_function(Bucket* p, va_list args)
 {
@@ -2137,6 +2329,150 @@ apc_optimize_function_t apc_register_optimizer(apc_optimize_function_t optimizer
 }
 /* }}} */
 
+#ifdef ZEND_ENGINE_2_4
+
+#define APC_COPY_TRAIT_METHOD(dst, src) \
+    CHECK(dst = \
+         (zend_trait_method_reference *)apc_pool_alloc(ctxt->pool, sizeof(zend_trait_method_reference))); \
+    memcpy(dst, src, sizeof(zend_trait_method_reference)); \
+\
+    if (src->method_name) { \
+        CHECK((dst->method_name = apc_pstrdup(src->method_name, ctxt->pool TSRMLS_CC))); \
+        dst->mname_len = src->mname_len; \
+    } \
+ \
+    if (src->class_name) { \
+        CHECK((dst->class_name = apc_pstrdup(src->class_name, ctxt->pool TSRMLS_CC))); \
+        dst->cname_len = src->cname_len; \
+    } \
+    if (src->ce) { \
+        dst->ce = my_copy_class_entry(NULL, src->ce, ctxt TSRMLS_CC); \
+    }
+
+#define APC_COPY_TRAIT_METHOD_FOR_EXEC(dst, src) \
+    dst = (zend_trait_method_reference *) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_method_reference)); \
+    memcpy(dst, src, sizeof(zend_trait_method_reference)); \
+    if (src->method_name) { \
+        CHECK((dst->method_name = apc_pstrdup(src->method_name, ctxt->pool TSRMLS_CC))); \
+    } \
+\
+    if (src->class_name) { \
+        CHECK((dst->class_name = apc_pstrdup(src->class_name, ctxt->pool TSRMLS_CC))); \
+    } \
+    if (src->ce) { \
+        dst->ce = apc_copy_class_entry_for_execution(src->ce, ctxt TSRMLS_CC); \
+    }
+
+/* {{{ apc_copy_trait_alias */
+zend_trait_alias* apc_copy_trait_alias(zend_trait_alias *dst, zend_trait_alias *src, apc_context_t *ctxt TSRMLS_DC)
+{
+    if (!dst) {
+        CHECK(dst = (zend_trait_alias *) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_alias)));
+    }
+    memcpy(dst, src, sizeof(zend_trait_alias));
+
+    if (src->alias) {
+        CHECK((dst->alias = apc_pstrdup(src->alias, ctxt->pool TSRMLS_CC)));
+        dst->alias_len = src->alias_len;
+    }
+
+    if (src->function) {
+        CHECK(dst->function = my_copy_function(NULL, src->function, ctxt TSRMLS_CC));
+    }
+
+    APC_COPY_TRAIT_METHOD(dst->trait_method, src->trait_method);
+
+    return dst;
+}
+/* }}} */
+
+/* {{{ apc_copy_trait_alias_for_execution */
+zend_trait_alias* apc_copy_trait_alias_for_execution(zend_trait_alias *src, apc_context_t *ctxt TSRMLS_DC)
+{
+    zend_trait_alias *dst = (zend_trait_alias *) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_alias));
+
+    memcpy(dst, src, sizeof(zend_trait_alias));
+
+    if (src->alias) {
+        CHECK((dst->alias = apc_pstrdup(src->alias, ctxt->pool TSRMLS_CC)));
+    }
+
+    /* XXX function is not handled here, check if it's right */
+
+    APC_COPY_TRAIT_METHOD_FOR_EXEC(dst->trait_method, src->trait_method);
+
+    return dst;
+}
+/* }}} */
+
+/* {{{ apc_copy_trait_precedence */
+zend_trait_precedence* apc_copy_trait_precedence(zend_trait_precedence *dst, zend_trait_precedence *src, apc_context_t *ctxt TSRMLS_DC)
+{
+    if (!dst) {
+        CHECK(dst = (zend_trait_precedence *) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_precedence)));
+    }
+    memcpy(dst, src, sizeof(zend_trait_precedence));
+
+    if (src->function) {
+        CHECK(dst->function = my_copy_function(NULL, src->function, ctxt TSRMLS_CC));
+    } 
+
+    if (src->exclude_from_classes && *src->exclude_from_classes) {
+        int i = 0, num_classes = 0;
+
+        while (src->exclude_from_classes[num_classes]) { num_classes++; }
+
+        CHECK(dst->exclude_from_classes =
+            (zend_class_entry **) apc_pool_alloc(ctxt->pool, sizeof(zend_class_entry *)*(num_classes+1)));
+
+        while (src->exclude_from_classes[i] && i < num_classes) {
+            char *name = (char *) src->exclude_from_classes[i];
+
+            dst->exclude_from_classes[i] = (zend_class_entry *) apc_pstrdup(name, ctxt->pool TSRMLS_CC);
+            i++;
+        }
+        dst->exclude_from_classes[i] = NULL;
+    }
+
+    APC_COPY_TRAIT_METHOD(dst->trait_method, src->trait_method);
+
+    return dst;
+}
+/* }}} */
+
+/* {{{ apc_copy_trait_precedence_for_execution */
+zend_trait_precedence* apc_copy_trait_precedence_for_execution(zend_trait_precedence *src, apc_context_t *ctxt TSRMLS_DC)
+{
+    zend_trait_precedence *dst = (zend_trait_precedence *) apc_pool_alloc(ctxt->pool, sizeof(zend_trait_precedence));
+
+    memcpy(dst, src, sizeof(zend_trait_precedence));
+
+    if (src->exclude_from_classes && *src->exclude_from_classes) {
+        int i = 0, num_classes = 0;
+
+        while (src->exclude_from_classes[num_classes]) { num_classes++; }
+
+        CHECK(dst->exclude_from_classes = 
+            (zend_class_entry **) apc_pool_alloc(ctxt->pool, sizeof(zend_class_entry *)*(num_classes+1)));
+
+        while (src->exclude_from_classes[i] && i < num_classes) {
+            char *name = (char *) src->exclude_from_classes[i];
+
+            dst->exclude_from_classes[i] = (zend_class_entry *) apc_pstrdup(name, ctxt->pool TSRMLS_CC);
+            i++;
+        }
+        dst->exclude_from_classes[i] = NULL;
+    }
+
+    /* XXX function is not handled here, check if it's right */
+
+    APC_COPY_TRAIT_METHOD(dst->trait_method, src->trait_method);
+
+    return dst;
+}
+/* }}} */
+#endif
+
 /*
  * Local variables:
  * tab-width: 4
