@@ -27,7 +27,7 @@
 
  */
 
-/* $Id: apc_compile.h 307185 2011-01-06 21:13:11Z gopalv $ */
+/* $Id: apc_compile.h 327059 2012-08-11 16:04:48Z ab $ */
 
 #ifndef APC_COMPILE_H
 #define APC_COMPILE_H
@@ -93,6 +93,10 @@ extern zend_class_entry* apc_copy_class_entry(zend_class_entry* dst, zend_class_
 extern apc_function_t* apc_copy_new_functions(int old_count, apc_context_t* ctxt TSRMLS_DC);
 extern apc_class_t* apc_copy_new_classes(zend_op_array* op_array, int old_count, apc_context_t* ctxt TSRMLS_DC);
 extern zval* apc_copy_zval(zval* dst, const zval* src, apc_context_t* ctxt TSRMLS_DC);
+#ifdef ZEND_ENGINE_2_4
+extern zend_trait_alias* apc_copy_trait_alias(zend_trait_alias *dst, zend_trait_alias *src, apc_context_t *ctxt TSRMLS_DC);
+extern zend_trait_precedence* apc_copy_trait_precedence(zend_trait_precedence *dst, zend_trait_precedence *src, apc_context_t *ctxt TSRMLS_DC);
+#endif
 
 #if 0
 /*
@@ -115,13 +119,19 @@ extern void apc_free_zval(zval* src, apc_context_t* ctxt);
 extern zend_op_array* apc_copy_op_array_for_execution(zend_op_array* dst, zend_op_array* src, apc_context_t* ctxt TSRMLS_DC);
 extern zend_function* apc_copy_function_for_execution(zend_function* src, apc_context_t* ctxt TSRMLS_DC);
 extern zend_class_entry* apc_copy_class_entry_for_execution(zend_class_entry* src, apc_context_t* ctxt TSRMLS_DC);
+#ifdef ZEND_ENGINE_2_4
+extern zend_trait_alias* apc_copy_trait_alias_for_execution(zend_trait_alias *src, apc_context_t *ctxt TSRMLS_DC);
+extern zend_trait_precedence* apc_copy_trait_precedence_for_execution(zend_trait_precedence *src, apc_context_t *ctxt TSRMLS_DC);
+#endif
 
 /*
- * The "free-after-execution" function performs a cursory clean up of the class data
+ * The "free-after-execution" function performs a cursory clean up of the item data
  * This is required to minimize memory leak warnings and to ensure correct destructor
  * ordering of some variables.
  */
 extern void apc_free_class_entry_after_execution(zend_class_entry* src TSRMLS_DC);
+extern void apc_free_function_after_execution(zend_function* src TSRMLS_DC);
+extern void apc_free_op_array_after_execution(zend_op_array *src TSRMLS_DC);
 
 /*
  * Optimization callback definition and registration function. 
