@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,18 +26,18 @@
 
  */
 
-/* $Id: apc_fcntl.h 268255 2008-11-04 05:42:11Z rasmus $ */
+/* $Id: apc_fcntl.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_FCNTL_H
 #define APC_FCNTL_H
 
 
-extern int apc_fcntl_create(const char* pathname);
+extern int apc_fcntl_create(const char* pathname TSRMLS_DC);
 extern void apc_fcntl_destroy(int fd);
-extern void apc_fcntl_lock(int fd);
-extern void apc_fcntl_rdlock(int fd);
-extern void apc_fcntl_unlock(int fd);
-extern unsigned char apc_fcntl_nonblocking_lock(int fd);
+extern void apc_fcntl_lock(int fd TSRMLS_DC);
+extern void apc_fcntl_rdlock(int fd TSRMLS_DC);
+extern void apc_fcntl_unlock(int fd TSRMLS_DC);
+extern unsigned char apc_fcntl_nonblocking_lock(int fd TSRMLS_DC);
 #endif
 
 /*
