@@ -27,7 +27,7 @@
 
  */
 
-/* $Id: apc_lock.h 309203 2011-03-14 06:47:16Z pajoye $ */
+/* $Id: apc_lock.h 311339 2011-05-22 17:18:49Z gopalv $ */
 
 #ifndef APC_LOCK
 #define APC_LOCK
@@ -154,7 +154,7 @@
 # define apc_lck_nb_lock(a)    apc_fcntl_nonblocking_lock(a TSRMLS_CC)
 # define apc_lck_rdlock(a)     apc_fcntl_rdlock(a TSRMLS_CC)
 # define apc_lck_unlock(a)     apc_fcntl_unlock(a TSRMLS_CC)
-# define apc_lck_rdunlock(a)   apc_fcntl_unlock(&a TSRMLS_CC)
+# define apc_lck_rdunlock(a)   apc_fcntl_unlock(a TSRMLS_CC)
 #endif
 
 #endif
