@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_main.c 309002 2011-03-07 19:50:18Z pajoye $ */
+/* $Id: apc_main.c 327102 2012-08-13 13:04:46Z ab $ */
 
 #include "apc_php.h"
 #include "apc_main.h"
@@ -90,7 +90,7 @@ static int install_function(apc_function_t fn, apc_context_t* ctxt, int lazy TSR
                               fn.name,
                               fn.name_len+1,
                               func,
-                              sizeof(fn.function[0]),
+                              sizeof(zend_function),
                               NULL);
         efree(func);
     }
@@ -133,7 +133,6 @@ static int install_class(apc_class_t cl, apc_context_t* ctxt, int lazy TSRMLS_DC
     zend_class_entry* class_entry = cl.class_entry;
     zend_class_entry* parent = NULL;
     int status;
-    zend_class_entry** allocated_ce = NULL;
 
     /* Special case for mangled names. Mangled names are unique to a file.
      * There is no way two classes with the same mangled name will occur,
@@ -162,16 +161,6 @@ static int install_class(apc_class_t cl, apc_context_t* ctxt, int lazy TSRMLS_DC
         return status;
     }
 
-    /*
-     * XXX: We need to free this somewhere...
-     */
-    allocated_ce = apc_php_malloc(sizeof(zend_class_entry*) TSRMLS_CC);
-
-    if(!allocated_ce) {
-        return FAILURE;
-    }
-
-    *allocated_ce =
     class_entry =
         apc_copy_class_entry_for_execution(cl.class_entry, ctxt TSRMLS_CC);
 
@@ -213,20 +202,19 @@ static int install_class(apc_class_t cl, apc_context_t* ctxt, int lazy TSRMLS_DC
             class_entry->parent = parent;
             zend_do_inheritance(class_entry, parent TSRMLS_CC);
         }
-
-
     }
 
     status = zend_hash_add(EG(class_table),
                            cl.name,
                            cl.name_len+1,
-                           allocated_ce,
+                           &class_entry,
                            sizeof(zend_class_entry*),
                            NULL);
 
     if (status == FAILURE) {
         apc_error("Cannot redeclare class %s" TSRMLS_CC, cl.name);
     }
+
     return status;
 }
 /* }}} */
@@ -374,6 +362,7 @@ static zend_op_array* cached_compile(zend_file_handle* h,
 
 default_compile:
 
+    /* XXX more cleanup in uninstall class, and uninstall_function() should be here too */
     if(cache_entry->data.file.classes) {
         for(ii = 0; ii < i ; ii++) {
             uninstall_class(cache_entry->data.file.classes[ii] TSRMLS_CC);
@@ -391,7 +380,7 @@ default_compile:
 /* }}} */
 
 /* {{{ apc_compile_cache_entry  */
-zend_bool apc_compile_cache_entry(apc_cache_key_t key, zend_file_handle* h, int type, time_t t, zend_op_array** op_array, apc_cache_entry_t** cache_entry TSRMLS_DC) {
+zend_bool apc_compile_cache_entry(apc_cache_key_t *key, zend_file_handle* h, int type, time_t t, zend_op_array** op_array, apc_cache_entry_t** cache_entry TSRMLS_DC) {
     int num_functions, num_classes;
     apc_function_t* alloc_functions;
     zend_op_array* alloc_op_array;
@@ -437,7 +426,7 @@ zend_bool apc_compile_cache_entry(apc_cache_key_t key, zend_file_handle* h, int
             while((n = php_stream_read(stream, (char*)buf, sizeof(buf))) > 0) {
                 PHP_MD5Update(&context, buf, n);
             }
-            PHP_MD5Final(key.md5, &context);
+            PHP_MD5Final(key->md5, &context);
             php_stream_close(stream);
             if(n<0) {
                 apc_warning("Error while reading '%s' for md5 generation." TSRMLS_CC, filename);
@@ -459,7 +448,7 @@ zend_bool apc_compile_cache_entry(apc_cache_key_t key, zend_file_handle* h, int
     }
 
     path = h->opened_path;
-    if(!path && key.type == APC_CACHE_KEY_FPFILE) path = (char*)key.data.fpfile.fullpath;
+    if(!path && key->type == APC_CACHE_KEY_FPFILE) path = (char*)key->data.fpfile.fullpath;
     if(!path) path=h->filename;
 
     apc_debug("2. h->opened_path=[%s]  h->filename=[%s]\n" TSRMLS_CC, h->opened_path?h->opened_path:"null",h->filename);
@@ -563,6 +552,10 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
             if (h->type != ZEND_HANDLE_FILENAME) {
                 zend_llist_add_element(&CG(open_files), h); 
             }
+
+            /* save this to free on rshutdown */
+            cache_entry->data.file.exec_refcount = op_array->refcount;
+
             return op_array;
         }
         if(APCG(report_autofilter)) {
@@ -607,7 +600,7 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
 #endif
 
     zend_try {
-        if (apc_compile_cache_entry(key, h, type, t, &op_array, &cache_entry TSRMLS_CC) == SUCCESS) {
+        if (apc_compile_cache_entry(&key, h, type, t, &op_array, &cache_entry TSRMLS_CC) == SUCCESS) {
             ctxt.pool = cache_entry->pool;
             ctxt.copy = APC_COPY_IN_OPCODE;
             if (apc_cache_insert(apc_cache, key, cache_entry, &ctxt, t TSRMLS_CC) != 1) {
@@ -699,7 +692,7 @@ static int apc_load_data(const char *data_file TSRMLS_DC)
 
         if(p) {
             p[0] = '\0';
-            key_len = strlen(key);
+            key_len = strlen(key)+1;
 
             data = data_unserialize(data_file TSRMLS_CC);
             if(data) {
@@ -823,6 +816,12 @@ int apc_module_init(int module_number TSRMLS_DC)
 
     apc_pool_init();
 
+#ifdef ZEND_ENGINE_2_4
+#ifndef ZTS
+    apc_interned_strings_init(TSRMLS_C);
+#endif
+#endif
+
     apc_data_preload(TSRMLS_C);
 
 #if APC_HAVE_LOOKUP_HOOKS
@@ -841,10 +840,6 @@ int apc_module_init(int module_number TSRMLS_DC)
     }
 #endif
 
-#ifdef ZEND_ENGINE_2_4
-    apc_interned_strings_init(TSRMLS_C);
-#endif
-
     APCG(initialized) = 1;
     return 0;
 }
@@ -887,14 +882,16 @@ int apc_module_shutdown(TSRMLS_D)
         apc_cache_release(apc_cache, cache_entry TSRMLS_CC);
     }
 
+#ifdef ZEND_ENGINE_2_4
+#ifndef ZTS
+    apc_interned_strings_shutdown(TSRMLS_C);
+#endif
+#endif
+
     apc_cache_destroy(apc_cache TSRMLS_CC);
     apc_cache_destroy(apc_user_cache TSRMLS_CC);
     apc_sma_cleanup(TSRMLS_C);
 
-#ifdef ZEND_ENGINE_2_4
-    apc_interned_strings_shutdown(TSRMLS_C);
-#endif
-
     APCG(initialized) = 0;
     return 0;
 }
@@ -913,7 +910,6 @@ int apc_process_shutdown(TSRMLS_D)
 }
 /* }}} */
 
-
 /* {{{ apc_deactivate */
 static void apc_deactivate(TSRMLS_D)
 {
@@ -922,14 +918,14 @@ static void apc_deactivate(TSRMLS_D)
      */
     while (apc_stack_size(APCG(cache_stack)) > 0) {
         int i;
-        zend_class_entry* zce = NULL;
-        void ** centry = (void*)(&zce);
-        zend_class_entry** pzce = NULL;
-
         apc_cache_entry_t* cache_entry =
             (apc_cache_entry_t*) apc_stack_pop(APCG(cache_stack));
 
         if (cache_entry->data.file.classes) {
+            zend_class_entry* zce = NULL;
+            void ** centry = (void*)(&zce);
+            zend_class_entry** pzce = NULL;
+
             for (i = 0; cache_entry->data.file.classes[i].class_entry != NULL; i++) {
                 centry = (void**)&pzce; /* a triple indirection to get zend_class_entry*** */
                 if(zend_hash_find(EG(class_table), 
@@ -950,8 +946,45 @@ static void apc_deactivate(TSRMLS_D)
                     cache_entry->data.file.classes[i].name_len+1);
 
                 apc_free_class_entry_after_execution(zce TSRMLS_CC);
+                zce = NULL;
             }
         }
+#if 0
+        if (cache_entry->data.file.functions) {
+            zend_function fn, *pfn = NULL;
+
+            for (i = 0; cache_entry->data.file.functions[i].function != NULL; i++) {
+
+                if (zend_hash_find(EG(function_table),
+                            cache_entry->data.file.functions[i].name,
+                            cache_entry->data.file.functions[i].name_len+1,
+                            (void**)&pfn) == FAILURE) {
+                    continue;
+                }
+
+                fn = *pfn;
+                zend_hash_del(EG(function_table),
+                        cache_entry->data.file.functions[i].name,
+                        cache_entry->data.file.functions[i].name_len+1);
+#if 0
+                apc_free_function_after_execution(&fn TSRMLS_CC);
+                efree(fn);
+#endif
+                pfn = NULL;
+            }
+        }
+
+        /* This is a very special case of apc_free_op_array_after_execution.
+           File related op_array->refcount is allocated on unpool for execution
+           and would never be freed in zend_execute_scripts() */
+#ifdef ZEND_ENGINE_2_4
+		if (cache_entry->data.file.exec_refcount) {
+			apc_php_free(cache_entry->data.file.exec_refcount TSRMLS_CC);
+			cache_entry->data.file.exec_refcount = NULL;
+		}
+#endif
+#endif
+
         apc_cache_release(apc_cache, cache_entry TSRMLS_CC);
     }
 }
@@ -994,6 +1027,7 @@ int apc_request_init(TSRMLS_D)
 
 int apc_request_shutdown(TSRMLS_D)
 {
+
 #if APC_HAVE_LOOKUP_HOOKS
     if(APCG(lazy_class_table)) {
         zend_hash_destroy(APCG(lazy_class_table));
@@ -1010,12 +1044,18 @@ int apc_request_shutdown(TSRMLS_D)
 #ifdef APC_FILEHITS
     zval_ptr_dtor(&APCG(filehits));
 #endif
+
+    /* As long as regex is compiled per request, it must be freed accordingly.*/
+    if (APCG(compiled_filters) && APCG(filters)) {
+        apc_regex_destroy_array(APCG(compiled_filters) TSRMLS_CC);
+        APCG(compiled_filters) = NULL;
+    }
+
     return 0;
 }
 
 /* }}} */
 
-
 /*
  * Local variables:
  * tab-width: 4
