@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 
  */
 
-/* $Id: apc_pthreadmutex.h 268255 2008-11-04 05:42:11Z rasmus $ */
+/* $Id: apc_pthreadmutex.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_PTHREADMUTEX_H
 #define APC_PTHREADMUTEX_H
@@ -28,11 +28,11 @@
 
 #include <pthread.h>
 
-pthread_mutex_t *apc_pthreadmutex_create();
+pthread_mutex_t *apc_pthreadmutex_create(pthread_mutex_t *lock TSRMLS_DC);
 void apc_pthreadmutex_destroy(pthread_mutex_t *lock);
-void apc_pthreadmutex_lock(pthread_mutex_t *lock);
-void apc_pthreadmutex_unlock(pthread_mutex_t *lock);
-zend_bool apc_pthreadmutex_nonblocking_lock(pthread_mutex_t *lock);
+void apc_pthreadmutex_lock(pthread_mutex_t *lock TSRMLS_DC);
+void apc_pthreadmutex_unlock(pthread_mutex_t *lock TSRMLS_DC);
+zend_bool apc_pthreadmutex_nonblocking_lock(pthread_mutex_t *lock TSRMLS_DC);
 
 #endif
 
