@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_sem.c 268620 2008-11-09 02:30:49Z shire $ */
+/* $Id: apc_sem.c 307048 2011-01-03 23:53:17Z kalle $ */
 
 #include "apc.h"
 
@@ -69,38 +69,29 @@ union semun {
 # define UNDO 0
 #endif
 
-int apc_sem_create(const char* pathname, int proj, int initval)
+int apc_sem_create(int proj, int initval TSRMLS_DC)
 {
     int semid;
-    int perms;
+    int perms = 0777;
     union semun arg;
-    key_t key;
-
-    perms = 0777;
-
-    key = IPC_PRIVATE;
-    if (pathname != NULL) {
-        if ((key = ftok(pathname, proj)) < 0) {
-            apc_eprint("apc_sem_create: ftok(%s,%d) failed:", pathname, proj);
-        }
-    }
+    key_t key = IPC_PRIVATE;
 
     if ((semid = semget(key, 1, IPC_CREAT | IPC_EXCL | perms)) >= 0) {
         /* sempahore created for the first time, initialize now */
         arg.val = initval;
         if (semctl(semid, 0, SETVAL, arg) < 0) {
-            apc_eprint("apc_sem_create: semctl(%d,...) failed:", semid);
+            apc_error("apc_sem_create: semctl(%d,...) failed:" TSRMLS_CC, semid);
         }
     }
     else if (errno == EEXIST) {
         /* sempahore already exists, don't initialize */
         if ((semid = semget(key, 1, perms)) < 0) {
-            apc_eprint("apc_sem_create: semget(%u,...) failed:", key);
+            apc_error("apc_sem_create: semget(%u,...) failed:" TSRMLS_CC, key);
         }
         /* insert <sleazy way to avoid race condition> here */
     }
     else {
-        apc_eprint("apc_sem_create: semget(%u,...) failed:", key);
+        apc_error("apc_sem_create: semget(%u,...) failed:" TSRMLS_CC, key);
     }
 
     return semid;
@@ -113,7 +104,7 @@ void apc_sem_destroy(int semid)
     semctl(semid, 0, IPC_RMID, arg);
 }
 
-void apc_sem_lock(int semid)
+void apc_sem_lock(int semid TSRMLS_DC)
 {
     struct sembuf op;
 
@@ -123,12 +114,12 @@ void apc_sem_lock(int semid)
 
     if (semop(semid, &op, 1) < 0) {
         if (errno != EINTR) {
-            apc_eprint("apc_sem_lock: semop(%d) failed:", semid);
+            apc_error("apc_sem_lock: semop(%d) failed:" TSRMLS_CC, semid);
         }
     }
 }
 
-int apc_sem_nonblocking_lock(int semid) 
+int apc_sem_nonblocking_lock(int semid TSRMLS_DC) 
 {
     struct sembuf op;
 
@@ -140,14 +131,14 @@ int apc_sem_nonblocking_lock(int semid)
       if (errno == EAGAIN) {
         return 0;  /* Lock is already held */
       } else if (errno != EINTR) {
-        apc_eprint("apc_sem_lock: semop(%d) failed:", semid);
+        apc_error("apc_sem_lock: semop(%d) failed:" TSRMLS_CC, semid);
       }
     }
 
     return 1;  /* Lock obtained */
 }
 
-void apc_sem_unlock(int semid)
+void apc_sem_unlock(int semid TSRMLS_DC)
 {
     struct sembuf op;
 
@@ -157,12 +148,12 @@ void apc_sem_unlock(int semid)
 
     if (semop(semid, &op, 1) < 0) {
         if (errno != EINTR) {
-            apc_eprint("apc_sem_unlock: semop(%d) failed:", semid);
+            apc_error("apc_sem_unlock: semop(%d) failed:" TSRMLS_CC, semid);
         }
     }
 }
 
-void apc_sem_wait_for_zero(int semid)
+void apc_sem_wait_for_zero(int semid TSRMLS_DC)
 {
     struct sembuf op;
 
@@ -172,19 +163,19 @@ void apc_sem_wait_for_zero(int semid)
 
     if (semop(semid, &op, 1) < 0) {
         if (errno != EINTR) {
-            apc_eprint("apc_sem_waitforzero: semop(%d) failed:", semid);
+            apc_error("apc_sem_waitforzero: semop(%d) failed:" TSRMLS_CC, semid);
         }
     }
 }
 
-int apc_sem_get_value(int semid)
+int apc_sem_get_value(int semid TSRMLS_DC)
 {
     union semun arg;
     unsigned short val[1];
 
     arg.array = val;
     if (semctl(semid, 0, GETALL, arg) < 0) {
-        apc_eprint("apc_sem_getvalue: semctl(%d,...) failed:", semid);
+        apc_error("apc_sem_getvalue: semctl(%d,...) failed:" TSRMLS_CC, semid);
     }
     return val[0];
 }
