@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,20 +25,20 @@
 
  */
 
-/* $Id: apc_sem.h 268620 2008-11-09 02:30:49Z shire $ */
+/* $Id: apc_sem.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_SEM_H
 #define APC_SEM_H
 
 /* Wrapper functions for SysV sempahores */
 
-extern int apc_sem_create(const char* pathname, int proj, int initval);
+extern int apc_sem_create(int proj, int initval TSRMLS_DC);
 extern void apc_sem_destroy(int semid);
-extern void apc_sem_lock(int semid);
-extern int apc_sem_nonblocking_lock(int semid); 
-extern void apc_sem_unlock(int semid);
-extern void apc_sem_wait_for_zero(int semid);
-extern int apc_sem_get_value(int semid);
+extern void apc_sem_lock(int semid TSRMLS_DC);
+extern int apc_sem_nonblocking_lock(int semid TSRMLS_DC); 
+extern void apc_sem_unlock(int semid TSRMLS_DC);
+extern void apc_sem_wait_for_zero(int semid TSRMLS_DC);
+extern int apc_sem_get_value(int semid TSRMLS_DC);
 
 #endif
 
